// Copyright (c) 2012 Bryan Schumaker.
#include <libsaria/library.h>
#include <libsaria/track.h>
#include <libsaria/print.h>
#include <libsaria/deck.h>
#include <libsaria/idle.h>
#include <libsaria/fs.h>
#include "library.h"

static unsigned int MAX_SCAN = 20;

/*
 * Scan the library path and remove files that don't exist anymore
 */
class ValidateTask : public IdleTask
{
	private:
		struct libsaria::library::Path *path;

	public:
		ValidateTask(struct libsaria::library::Path *);
		~ValidateTask();
		void run_task();
};

ValidateTask::ValidateTask(struct libsaria::library::Path *p)
{
	path = p;
}

ValidateTask::~ValidateTask()
{
}

void ValidateTask::run_task()
{
	list<libsaria::Track> to_rm;
	list<libsaria::Track>::iterator it, it_mv;

	for (it = path->tracks.begin(); it != path->tracks.end(); it++) {
		if (!libsaria::exists(it->get_filepath())) {
			println("Reaping: " + it->get_filepath());
			/* Iterators are no longer valid after splice */
			it_mv = it;
			it--;
			to_rm.splice(to_rm.begin(), path->tracks, it_mv);
			path->data_state = DIRTY;
		}
	};

	libsaria::deck::tracks_removed(to_rm);
	notify_path_updated(path);
	save_path(path);
}


/*
 * Tag each file in file_list and add to the library path
 */
class ScanTask : public IdleTask
{
	private:
		struct libsaria::library::Path *path;
		list<string> file_list;

	public:
		ScanTask(struct libsaria::library::Path *, list<string> &);
		~ScanTask();
		void run_task();
};

ScanTask::ScanTask(struct libsaria::library::Path *p, list<string> &files)
{
	list<string>::iterator it;
	path = p;

	if (files.size() <= MAX_SCAN)
		it = files.end();
	else {
		it = files.begin();
		for (unsigned int i = 0; i < MAX_SCAN; i++)
			it++;
	}

	file_list.splice(file_list.begin(), files, files.begin(), it);
}

ScanTask::~ScanTask()
{
}

static bool new_file(string &file, list<libsaria::Track> &tracks)
{
	list<libsaria::Track>::iterator it;
	for (it = tracks.begin(); it != tracks.end(); it++) {
		if ((*it).get_filepath() == file)
			return false;
	}
	return true;
}

void ScanTask::run_task()
{
	list<string>::iterator it;
	list<libsaria::Track *> tracks;

	for (it = file_list.begin(); it != file_list.end(); it++) {
		if (!new_file(*it, path->tracks))
			continue;

		try {
			path->tracks.push_back(libsaria::Track(*it, path));
			tracks.push_back(&path->tracks.back());
			path->data_state = DIRTY;
		} catch (string message) {
			println(message);
		}
	}

	notify_path_updated(path);
	lib_playlist.add_tracks(tracks);
}


/*
 * Read a directory and create ScanTasks for chunks of files
 */
class ReaddirTask : public IdleTask
{
	private:
		struct libsaria::library::Path *path;

	public:
		ReaddirTask(struct libsaria::library::Path *path);
		~ReaddirTask();
		void run_task();
};

ReaddirTask::ReaddirTask(struct libsaria::library::Path *p)
{
	path = p;
}

ReaddirTask::~ReaddirTask()
{
}

void ReaddirTask::run_task()
{
	list<string> file_list;
	list<string>::iterator it;
	unsigned int i;
	ScanTask *task;

	libsaria::list_dir(path->path, file_list);
	println("Found: %d files to scan", file_list.size());

	i = 0;
	while (file_list.size() > 0) {
		task = new ScanTask(path, file_list);
		task->queue();
		if ((i++ % 50) == 0)
			save_path(path);
	}
	save_path(path);
}

void do_update_path(struct libsaria::library::Path *path)
{
	println("Updating path: " + path->path);

	if (!libsaria::exists(path->path)) {
		libsaria::library::delete_path(path);
		return;
	}

	ValidateTask *validate = new ValidateTask(path);
	ReaddirTask *task = new ReaddirTask(path);
	validate->queue();
	task->queue();
}
