// Copyright (c) 2012 Bryan Schumaker.
#include <libsaria/playlist.h>
#include <libsaria/audio.h>
#include <libsaria/track.h>
#include <libsaria/print.h>
#include <libsaria/deck.h>

#include <list>
#include <sstream>
using namespace std;

static void (*on_new_playlist)(libsaria::Playlist *) = NULL;

static AutoPauseType pause_type;
static unsigned short pause_count = 0;

static list<libsaria::Playlist *> playlist_deck;
static libsaria::List recent_plist("Recent", PL_STATIC | PL_NO_DRAIN);

static void renumber_playlists()
{
	list<libsaria::Playlist *>::iterator it;
	int n = 0;

	for (it = playlist_deck.begin(); it != playlist_deck.end(); it++)
			(*it)->renumber(n++);
}

static libsaria::Playlist *get_active_playlist()
{
	list<libsaria::Playlist *>::iterator it;

	for (it = playlist_deck.begin(); it != playlist_deck.end(); it++) {
		if (!(*it)->is_disabled())
			return *it;
	}

	return libsaria::library::get_playlist();
}

static bool check_play()
{
	bool ret = true;

	switch(pause_type) {
	case PS_AFTER_N:
		if (pause_count == 0) {
			ret = false;
			pause_type = PS_NONE;
		}
		pause_count--;
	default:
		break;
	}

	return ret;
}

namespace libsaria
{

	void deck::push_front(Playlist *plist)
	{
		playlist_deck.push_front(plist);
	}

	void deck::push_back(Playlist *plist)
	{
		playlist_deck.push_back(plist);
	}

	void deck::garbage_collect()
	{
		int n = 0;
		list<libsaria::Playlist *>::iterator it;

		for (it = playlist_deck.begin(); it != playlist_deck.end(); it++) {
			if ((*it)->get_size() == 0) {
				(*it)->prepare_for_removal();
				delete (*it);
				it = playlist_deck.erase(it);
				it--;
			} else
				(*it)->renumber(n++);
		}
	}

	void deck::next()
	{
		Playlist *plist = get_active_playlist();
		Track *track = plist->next();

		if (track)
			track->load(check_play());

		garbage_collect();
	}

	void deck::prev()
	{
		Track *track = recent_plist.next();
		if (track)
			track->load_unlisted(true);
	}

	void set_on_new_playlist(void (*func)(Playlist *))
	{
		on_new_playlist = func;
	}

	Playlist *deck::new_playlist(list<Track *> &tracks, PlaylistType type, bool front)
	{
		Playlist *plist;

		if (playlist_deck.size() == MAX_PLAYLISTS)
			return NULL;

		switch (type) {
		case PLIST_SET:
			plist = new Set(PL_NONE);
			break;
		case PLIST_QUEUE:
			plist = new Queue(PL_NONE);
			break;
		default:
			plist = new List(PL_NONE);
		};

		if (front)
			deck::push_front(plist);
		else
			deck::push_back(plist);
		renumber_playlists();

		if (on_new_playlist)
			on_new_playlist(plist);
		plist->add_tracks(tracks);
		return plist;
	}

	void deck::delete_playlist(Playlist *plist)
	{
		list<libsaria::Playlist *>::iterator it;
		for (it = playlist_deck.begin(); (*it) != plist; it++);

		plist->prepare_for_removal();
		delete plist;

		playlist_deck.erase(it);
		garbage_collect();
	}

	unsigned int deck::move_playlist(Playlist *plist, unsigned int index)
	{
		list<Playlist *>::iterator it;

		if (index >= playlist_deck.size())
			index = playlist_deck.size() - 1;

		playlist_deck.remove(plist);

		it = playlist_deck.begin();
		for (unsigned int i = 0; i < index; i++)
			it++;

		playlist_deck.insert(it, plist);
		garbage_collect();

		return index;
	}

	void deck::init()
	{
		app::mkdir("playlist");
	}

	void deck::list_recent(Track *track)
	{
		list<libsaria::Track *> tracks;
		tracks.push_back(track);
		recent_plist.add_tracks(tracks);
		recent_plist.reset_iterator();
	}

	Playlist *deck::get_recent_plist()
	{
		return &recent_plist;
	}

	Playlist *deck::get_playlist(unsigned int n)
	{
		list<libsaria::Playlist *>::iterator it;

		if (n >= playlist_deck.size())
			return NULL;

		it = playlist_deck.begin();
		for (unsigned int i = 0; i < n; i++)
			it++;

		return *it;
	}

	void deck::tracks_removed(list<Track> &tracks)
	{
		list<Track *> track_ptrs;
		list<Track>::iterator t_it;
		list<Playlist *>::iterator it;

		for (t_it = tracks.begin(); t_it != tracks.end(); t_it++)
			track_ptrs.push_back(&(*t_it));

		for (it = playlist_deck.begin(); it != playlist_deck.end(); it++)
			(*it)->remove_tracks(track_ptrs);
	}

	void deck::set_pause_type(AutoPauseType type, unsigned int count)
	{
		pause_type = type;
		pause_count = count;
	}

	AutoPauseType deck::get_pause_type()
	{
		return pause_type;
	}

	unsigned short deck::get_pause_count()
	{
		return pause_count;
	}

}; /* Namespace: libsaria */
