// Copyright (c) 2012 Bryan Schumaker
#include <ocarina/ocarina.h>
#include <ocarina/button.h>
#include <ocarina/body.h>

#include <libsaria/audio.h>
#include <libsaria/deck.h>

static GtkWidget *notebook;
static GtkWidget *idle_progress;
static GtkWidget *pos_label;
static GtkWidget *track_progress;
static GtkWidget *dur_label;
static GtkWidget *play_button;
static GtkWidget *pause_button;
static GtkWidget *stop_button;
static GtkWidget *prev_button;
static GtkWidget *next_button;
static GtkWidget *length_label;
static GstState last_state;

static void add_page(string text, GtkWidget *page)
{
	GtkWidget *label = gtk_label_new(text.c_str());

	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), page, label);
	gtk_container_child_set(GTK_CONTAINER(notebook), page, "tab-expand", TRUE, NULL);
	gtk_container_child_set(GTK_CONTAINER(notebook), page, "tab-fill",   TRUE, NULL);
	gtk_widget_show(label);
}

static void update_progress()
{
	gint64 pos = libsaria::audio::position();
	gint64 dur = libsaria::audio::duration() + 1;

	gtk_label_set_text(GTK_LABEL(pos_label), libsaria::audio::posstr().c_str());
	gtk_label_set_text(GTK_LABEL(dur_label), libsaria::audio::durstr().c_str());

	/*
	 * This happens when gstreamer's "about-to-finish" signal
	 * is emitted and we queue up a shorter song before the
	 * pipeline finishes.
	 */
	if (pos > dur)
		pos = 0;
	gtk_range_set_range(GTK_RANGE(track_progress), 0, dur);
	gtk_range_set_value(GTK_RANGE(track_progress), pos);
}

static void slider_changed(GtkWidget *w, GtkScrollType s, gdouble v, gpointer d)
{
	libsaria::audio::seek_to(v);
}

static void on_click_play(GtkWidget *b, gpointer d)
{
	libsaria::audio::play();
}

static void on_click_pause(GtkWidget *b, gpointer d)
{
	libsaria::audio::pause();
}

static void on_click_stop(GtkWidget *b, gpointer d)
{
	libsaria::audio::stop();
}

static void on_click_prev(GtkWidget *b, gpointer d)
{
	libsaria::deck::prev();
}

static void on_click_next(GtkWidget *b, gpointer d)
{
	libsaria::deck::next();
}

static GtkWidget *make_controls()
{
	GtkWidget *box = gtk_hbox_new(FALSE, 0);

	play_button  = make_button(GTK_STOCK_MEDIA_PLAY, on_click_play, true);
	pause_button = make_button(GTK_STOCK_MEDIA_PAUSE, on_click_pause, false);
	stop_button  = make_button(GTK_STOCK_MEDIA_STOP, on_click_stop, true);
	prev_button  = make_button(GTK_STOCK_MEDIA_PREVIOUS, on_click_prev, true);
	next_button  = make_button(GTK_STOCK_MEDIA_NEXT, on_click_next, true);

	pos_label = gtk_label_new("");
	track_progress = gtk_hscale_new_with_range(0, 1, 1000000000);
	g_signal_connect(track_progress, "change-value", 
			 G_CALLBACK(slider_changed), NULL);
	dur_label = gtk_label_new("");
	gtk_scale_set_draw_value(GTK_SCALE(track_progress), FALSE);

	gtk_box_pack_start(GTK_BOX(box), pos_label,     FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(box), track_progress, TRUE,  TRUE, 0);
	gtk_box_pack_start(GTK_BOX(box), dur_label,     FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(box), play_button,   FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(box), pause_button,  FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(box), stop_button,   FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(box), prev_button,   FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(box), next_button,   FALSE, FALSE, 0);

	gtk_widget_show(pos_label);
	gtk_widget_show(dur_label);
	gtk_widget_show(track_progress);
	gtk_widget_show(box);
	return box;
}

namespace ocarina
{

	GtkWidget *body::footer_init()
	{
		GtkWidget *footer = gtk_vbox_new(FALSE, 0);
		GtkWidget *bottom_box = gtk_hbox_new(FALSE, 0);
		GtkWidget *controls = make_controls();
		notebook = gtk_notebook_new();

		idle_progress = gtk_progress_bar_new();
		gtk_rc_parse_string(
			"style \"idle-progress-style\"\n"
			"{\n"
			"	GtkProgressBar::min-horizontal-bar-height = 10\n"
			"	GtkProgressBar::min-horizontal-bar-width  = 75\n"
			"}\n"
			"widget \"*.ocarina-idle-progress\" style \"idle-progress-style\"");
		gtk_widget_set_name(idle_progress, "ocarina-idle-progress");

		length_label = gtk_label_new("");
		gtk_misc_set_alignment(GTK_MISC(length_label), 1.0, 0.0);
		gtk_box_pack_start(GTK_BOX(bottom_box), idle_progress, FALSE, FALSE, 0);
		gtk_box_pack_start(GTK_BOX(bottom_box), length_label, TRUE, TRUE, 0);

		gtk_notebook_set_tab_pos(GTK_NOTEBOOK(notebook), GTK_POS_TOP);
		g_object_set(notebook, "tab-border", 0, NULL);

		gtk_box_pack_start(GTK_BOX(footer), controls, FALSE, FALSE, 0);
		gtk_box_pack_start(GTK_BOX(footer), notebook, FALSE, FALSE, 0);
		gtk_box_pack_start(GTK_BOX(footer), bottom_box, FALSE, FALSE, 0);

		add_page("Now Playing", now_playing_page());
		add_page("AutoPause", pause_page());
		add_page("Library", library_page());
		add_page("Settings", general_page());

		gtk_widget_show(footer);
		gtk_widget_show(notebook);
		gtk_widget_show(bottom_box);
		gtk_widget_show(length_label);
		return footer;
	};

	void body::set_idle_progress(float progress)
	{
		gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(idle_progress), progress);
		gtk_widget_show(idle_progress);
	}

	void body::hide_idle_progress()
	{
		gtk_widget_hide(idle_progress);
	}

	void body::update_controls()
	{
		GstState cur_state;

		update_progress();

		cur_state = libsaria::audio::get_state();
		if (cur_state == last_state)
			return;

		last_state = cur_state;
		if (cur_state == GST_STATE_PLAYING) {
			gtk_widget_show(pause_button);
			gtk_widget_hide(play_button);
		} else {
			gtk_widget_show(play_button);
			gtk_widget_hide(pause_button);
		}
	}

	void body::set_length_label_text(string &text)
	{
		gtk_label_set_text(GTK_LABEL(length_label), text.c_str());
	}

}; /* Namespace: ocarina */
