// Copyright (c) 2012 Bryan Schumaker
#include <ocarina/playlist.h>
#include <ocarina/button.h>
#include <ocarina/body.h>

#include <libsaria/format.h>
#include <libsaria/track.h>
#include <libsaria/deck.h>

#include <sstream>
using namespace std;

static void disable_playlist(GtkWidget *button, gpointer d)
{
	libsaria::Playlist *plist = (libsaria::Playlist *)d;
	plist->set_disabled(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button)));
}

static void delete_playlist(GtkWidget *button, gpointer d)
{
	libsaria::Playlist *plist = (libsaria::Playlist *)d;
	libsaria::deck::delete_playlist(plist);
}

static void selected_foreach_list(GtkTreeModel *model, GtkTreePath *path,
				  GtkTreeIter  *iter,  gpointer data)
{
	libsaria::Track *track;
	list<libsaria::Track *> *tracks = (list<libsaria::Track *> *)data;

	gtk_tree_model_get(model, iter, 0, &track, -1);
	tracks->push_back(track);
}

static void selected_foreach_index(GtkTreeModel *model, GtkTreePath *path,
				   GtkTreeIter  *iter,  gpointer data)
{
	list<unsigned int> *indices = (list<unsigned int> *)data;
	unsigned int *index = (unsigned int *)gtk_tree_path_get_indices(path);
	indices->push_back(*index);
}

static void do_filter(GtkWidget *entry, gpointer d)
{
	string text = gtk_entry_get_text(GTK_ENTRY(entry));
	ocarina::Playlist *plist = (ocarina::Playlist *)d;
	plist->get_playlist()->set_filter_text(text);
}

namespace ocarina
{

	Playlist::Playlist(unsigned int f) : libsaria::PlaylistRenderer(f)
	{
		visible = false;
	}

	Playlist::Playlist(libsaria::Playlist *plist): libsaria::PlaylistRenderer(PL_NONE)
	{
		set_playlist(plist);
	}

	Playlist::~Playlist()
	{
	}

	void Playlist::set_length_label_text()
	{
		string text;
		stringstream stream;
		stream << playlist->get_size() << " ";
		stream << "songs: " << libsaria::length_string(playlist->get_length());

		text = stream.str();
		body::set_length_label_text(text);
	}

	void Playlist::set_label_text()
	{
		string text;
		stringstream stream;

		stream << playlist->get_name() << "\n";
		stream << playlist->get_size();
		text = stream.str();

		gtk_label_set_text(GTK_LABEL(name), text.c_str());
		if (visible)
			set_length_label_text();
	}

	void Playlist::list_selected_tracks(list<libsaria::Track *> &tracks)
	{
		gtk_tree_selection_selected_foreach(treesel, selected_foreach_list, &tracks);
	}

	void Playlist::list_selected_indices(list<unsigned int> &indices)
	{
		gtk_tree_selection_selected_foreach(treesel, selected_foreach_index, &indices);
	}

	void Playlist::select_none()
	{
		gtk_tree_selection_unselect_all(treesel);
	}

	void Playlist::switch_to()
	{
		body::switch_to_page(box);
	}

	void Playlist::on_show()
	{
		visible = true;
		set_length_label_text();
	}

	void Playlist::on_hide()
	{
		visible = false;
	}

	void Playlist::goto_index(unsigned int index)
	{
		GtkTreeIter iter;
		GtkTreePath *path;
		GtkTreeViewColumn *col;
		GdkRectangle area;
		unsigned int height, n;

		if (!visible)
			return;

		gtk_tree_model_get_iter_first(GTK_TREE_MODEL(liststore), &iter);
		path = gtk_tree_model_get_path(GTK_TREE_MODEL(liststore), &iter);
		col  = gtk_tree_view_get_column(GTK_TREE_VIEW(treeview), 0);

		gtk_tree_view_get_visible_rect(GTK_TREE_VIEW(treeview), &area);
		height = area.height;
		gtk_tree_view_get_cell_area(GTK_TREE_VIEW(treeview), path, col, &area);
		n = height / area.height;

		if (index < (n / 2))
			n = 0;
		else
			n = index - (n / 2);

		for (unsigned int i = 0; i < n; i++)
			gtk_tree_path_next(path);
		gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(treeview), path, NULL, TRUE, 0, 0);

		for (unsigned int i = 0; i < (index - n); i++)
			gtk_tree_path_next(path);
		gtk_tree_view_set_cursor_on_cell(GTK_TREE_VIEW(treeview), path, NULL, NULL, FALSE);

		gtk_tree_path_free(path);
	}

	void Playlist::set_playlist(libsaria::Playlist *p)
	{
		GtkWidget *label_box = gtk_hbox_new(FALSE, 0);
		GtkWidget *entry_box = gtk_hbox_new(FALSE, 0);
		GtkWidget *close_button;

		libsaria::PlaylistRenderer::set_playlist(p);

		number = gtk_label_new("");
		name   = gtk_label_new("");
		box    = gtk_vbox_new(FALSE, 0);

		gtk_label_set_justify(GTK_LABEL(name), GTK_JUSTIFY_RIGHT);
		gtk_misc_set_alignment(GTK_MISC(name), 1.0, 0.0);

		entry = gtk_entry_new();
		g_signal_connect(entry, "changed", G_CALLBACK(do_filter), this);

		gtk_box_pack_start(GTK_BOX(entry_box), entry, TRUE, TRUE, 0);
		gtk_box_pack_start(GTK_BOX(box), entry_box, FALSE, FALSE, 0);

		setup_treeview();

		gtk_box_pack_start(GTK_BOX(label_box), number, FALSE, FALSE, 0);
		gtk_box_pack_start(GTK_BOX(label_box), name,    TRUE,  TRUE, 0);

		if (!p->is_static()) {
			/* Make disable button for playlists */
			disable_button = make_toggle_button_data(GTK_STOCK_YES,
								 disable_playlist,
								 p, true);
			gtk_box_pack_start(GTK_BOX(entry_box), disable_button, FALSE, FALSE, 0);

			/* Make close button for tabs */
			close_button = make_button_data(GTK_STOCK_CLOSE, delete_playlist, p, true);
			gtk_button_set_relief(GTK_BUTTON(close_button), GTK_RELIEF_NONE);
			gtk_rc_parse_string(
				"style \"close-button-style\"\n"
				"{\n"
				"	GtkButton::inner-border = {0,0,0,0}\n"
				"}\n"
				"widget \"*.ocarina-close-button\" style \"close-button-style\"");
			gtk_widget_set_name(close_button, "ocarina-close-button");
			gtk_box_pack_start(GTK_BOX(label_box), close_button, FALSE, FALSE, 0);
		}

		gtk_widget_show_all(label_box);
		gtk_widget_show_all(entry_box);

		set_label_text();
		renumbered(p->get_number());
		body::add_page(p->get_number(), box, label_box, p);
	}

	void Playlist::renumbered(int n)
	{
		stringstream s;
		if (n == -1)
			return;
		s << "<span size='x-large'>" << n << " </span>";
		gtk_label_set_markup(GTK_LABEL(number), s.str().c_str());
	}

	void Playlist::prepare_for_removal()
	{
		body::remove_page(box);
	}

	void Playlist::on_disable(bool disabled)
	{
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(disable_button), disabled);
		gtk_widget_set_sensitive(entry, !disabled);
		gtk_widget_set_sensitive(treeview, !disabled);
		gtk_widget_set_sensitive(name, !disabled);
		gtk_widget_set_sensitive(number, !disabled);

		if (disabled)
			change_button_image(disable_button, GTK_STOCK_NO);
		else
			change_button_image(disable_button, GTK_STOCK_YES);
	}

}; /* Namespace: ocarina */
